/* 4+1 Joystick plus Simon game
 * Copyright (C) 2015 David Guerrero
 * Based on Teensy Gamepad for Josh Kropf <josh@slashdev.ca>
 * Based on keyboard example for Teensy USB Development Board
 * http://www.pjrc.com/teensy/usb_keyboard.html
 * Copyright (c) 2008 PJRC.COM, LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#ifndef HD44780_H_INCLUDED
#include "HD44780.h"
#endif

#include "usb_gamepad.h"

#ifndef TIMING
#include "timing.h"
#endif

#ifndef SOUND
#include "sound.h"
#endif

#ifndef CENTRAL_BUTTONS
#include "central_buttons.h"
#endif

#ifndef SIMON
#include "simon.h"
#endif

#define MASTER_GAMEPAD_ID (NUMBER_OF_INTERFACES-1)

#define LED_CONFIG  (DDRD |= (1<<6))
#define LED_OFF     (PORTD &= ~(1<<6))
#define LED_ON      (PORTD |= (1<<6))

#define CPU_PRESCALE(n)	(CLKPR = 0x80, CLKPR = (n))

#define DIRECTION_DDR DDRC
#define DIRECTION_PORT PORTC
#define DIRECTION_PINS PINC

#define UP_PIN 1
#define DOWN_PIN 3
#define LEFT_PIN 0
#define RIGHT_PIN 2
//#define DIRECTION_PINS_MASK ((1<<UP_PIN)|(1<<DOWN_PIN)|(1<<RIGHT_PIN)|(1<<LEFT_PIN))

#define SELECTOR_DDR DDRB
#define SELECTOR_PORT PORTB
#define SELECTOR_PINS PINB
#define SELECTOR_PINS_MASK (((1<<NUMBER_OF_INTERFACES)-1)<<1)//player selection pins starting at 1

#define is_active_pin(port,pin) (!(port&(1<<pin)))

#define BUTTONS_DDR	DDRF
#define BUTTONS_PORT	PORTF
#define BUTTONS_PINS	PINF

#define MASTER_BUTTON_0_DDR DDRD
#define MASTER_BUTTON_0_PORT PORTD
#define MASTER_BUTTON_0_PINS PIND
#define MASTER_BUTTON_0_PIN 0

#define MASTER_BUTTON_1_DDR DDRD
#define MASTER_BUTTON_1_PORT PORTD
#define MASTER_BUTTON_1_PINS PIND
#define MASTER_BUTTON_1_PIN 5

#define MASTER_BUTTON_2_DDR DDRD
#define MASTER_BUTTON_2_PORT PORTD
#define MASTER_BUTTON_2_PINS PIND
#define MASTER_BUTTON_2_PIN 7

#define MASTER_BUTTON_3_DDR DDRE
#define MASTER_BUTTON_3_PORT PORTE
#define MASTER_BUTTON_3_PINS PINE
#define MASTER_BUTTON_3_PIN 0

#define MASTER_BUTTON_4_DDR DDRE
#define MASTER_BUTTON_4_PORT PORTE
#define MASTER_BUTTON_4_PINS PINE
#define MASTER_BUTTON_4_PIN 1

#define MASTER_BUTTON_5_DDR DDRB
#define MASTER_BUTTON_5_PORT PORTB
#define MASTER_BUTTON_5_PINS PINB
#define MASTER_BUTTON_5_PIN 5

uint8_t axis_value(uint8_t port,uint8_t decrement_pin, uint8_t increment_pin)
{
	if(is_active_pin(port,increment_pin))
		return 0xff;
	if(is_active_pin(port,decrement_pin))
		return 0;
	return 0x80;
}

//volatile uint8_t scaned_gamepad=0;
volatile uint8_t light_buttons_values=0;
volatile uint8_t simon_start_buttons_values=0;
volatile uint8_t random_value=0;;

int32_t credits=0;
#define infinite_credits (credits<0)

void usb_gamepad_reset_state(gamepad_state_t* gamepad)
{
	(*gamepad).buttons=0;
	(*gamepad).higher_buttons=0;
	(*gamepad).y_axis =	0x80;
	(*gamepad).x_axis = 0x80;
}

void configure_polling_interrupt(void)
{
TCCR0A=2;//clear counter on compare match
OCR0A=62; //compare match when counter=62 (1000 times per second aprox.)=>in 5ms al devices are scanned.
TCNT0=0x00;     // set timer0 counter initial value to 0
TIMSK0=2;// enable timer 0 output compare match 2
TCCR0B=4;//divide frequency by 256
sei(); // enable interrupts
}

void display_available_credits()
{
	LCD_Clear();
	LCD_ClearLine(0);
	LCD_GotoXY(0,0);
	LCD_PrintString("credits / coins:");
	LCD_ClearLine(1);
	LCD_GotoXY(0,1);
	if(infinite_credits)
		LCD_PrintString("infinite");
	else
		LCD_PrintInteger(credits);
	if(credits)
		turn_on_all_color_button_lights;
	else
		turn_off_color_button_lights;
}

void increase_credits(int32_t increment)
{
	if(!infinite_credits)
		credits+=increment;
	display_available_credits();
}

int main(void) {
	CPU_PRESCALE(0); 	// set for 16 MHz clock
	configure_clock();
	LCD_Setup();
	configure_beeper();
	configure_central_buttons();
	configure_simon();

	LCD_LED_ON;
	LED_CONFIG;
	BUTTONS_DDR=  0;DIRECTION_DDR=  0; //set as input
	BUTTONS_PORT=  0xff; DIRECTION_PORT= 0xff; MCUCR&=~(1<<4); //activate pull-up
	SELECTOR_DDR|=SELECTOR_PINS_MASK;//set as output
	SELECTOR_PORT|=SELECTOR_PINS_MASK;//no gamepad selected

	MASTER_BUTTON_0_PORT|=1<<MASTER_BUTTON_0_PIN; //activate pull-up
	MASTER_BUTTON_1_PORT|=1<<MASTER_BUTTON_1_PIN; //activate pull-up
	MASTER_BUTTON_2_PORT|=1<<MASTER_BUTTON_2_PIN; //activate pull-up
	MASTER_BUTTON_3_PORT|=1<<MASTER_BUTTON_3_PIN; //activate pull-up
	MASTER_BUTTON_4_PORT|=1<<MASTER_BUTTON_4_PIN; //activate pull-up
	MASTER_BUTTON_5_PORT|=1<<MASTER_BUTTON_5_PIN; //activate pull-up

	for(uint8_t i=0;i<NUMBER_OF_INTERFACES;i++)
		usb_gamepad_reset_state(&gamepad_state[i]); //players will be reported as idle by default

	LED_ON; // power up led on startup for 1 sec

	// Initialize the USB, and then wait for the host to set configuration.
	// If the Teensy is powered without a PC connected to the USB port,
	// this will wait forever.
	usb_init();
	while (!usb_configured()) /* wait */ ;
	wait_for_miliseconds(1000);
	// Wait an extra second for the PC's operating system to load drivers
	// and do whatever it does to actually be ready for input
	configure_polling_interrupt();
	LED_OFF;
	display_available_credits();
	while (1)
	{
		for(uint8_t i=0;i<4;i++)
			if(pressed_simon_start_button(i))
				{
					increase_credits(simon_game(i+1));
				}
		for(uint8_t i=0;i<4;i++)
					if(pressed_light_button(i))//insert coin request
						{
							if(!credits)
								increase_credits(simon_game(0));
							else// a coin must be inserted
							{
										gamepad_state[MASTER_GAMEPAD_ID].higher_buttons=1<<i;
										if(!infinite_credits)
											increase_credits(-1);
										wait_for_miliseconds(50);//insert coin pulse duration
										gamepad_state[MASTER_GAMEPAD_ID].higher_buttons=0;
							}
							while(pressed_any_light_button);
							skip_bounces;
						}
	}
}

ISR(TIMER0_COMPA_vect)
{
	//read_gamepad_state();
	static uint8_t scaned_gamepad=MASTER_GAMEPAD_ID;
	uint8_t selected_button_mask=1<<scaned_gamepad;
	uint8_t next_gamepad= (scaned_gamepad==MASTER_GAMEPAD_ID) ? 0 : scaned_gamepad+1;
	gamepad_state_t* target=&gamepad_state[scaned_gamepad];
	if(scaned_gamepad!=MASTER_GAMEPAD_ID)//is a player joystick device
	{
		(*target).y_axis=axis_value(DIRECTION_PINS,DOWN_PIN,UP_PIN);
		(*target).x_axis=axis_value(DIRECTION_PINS,RIGHT_PIN,LEFT_PIN);
		(*target).buttons=(~BUTTONS_PINS);
		(*target).higher_buttons=(~DIRECTION_PINS)>>4;
		//now the state of the color central buttons and the simon start buttons must be updated
	 	if(color_button_pin_is_active)
			light_buttons_values|=selected_button_mask;
		else
			light_buttons_values&=~selected_button_mask;
	 	if(simon_start_button_pin_is_active)
			simon_start_buttons_values|=selected_button_mask;
		else
			simon_start_buttons_values&=~selected_button_mask;
	}
	else
	{
		(*target).buttons=0;//buttons 0-7 of the master interface are 0 by default
		if(is_active_pin(MASTER_BUTTON_0_PINS,MASTER_BUTTON_0_PIN))
			(*target).buttons|=1<<0;
		if(is_active_pin(MASTER_BUTTON_1_PINS,MASTER_BUTTON_1_PIN))
			(*target).buttons|=1<<1;
		if(is_active_pin(MASTER_BUTTON_2_PINS,MASTER_BUTTON_2_PIN))
			(*target).buttons|=1<<2;
		if(is_active_pin(MASTER_BUTTON_3_PINS,MASTER_BUTTON_3_PIN))
			(*target).buttons|=1<<3;
		if(is_active_pin(MASTER_BUTTON_4_PINS,MASTER_BUTTON_4_PIN))
			(*target).buttons|=1<<4;
		if(is_active_pin(MASTER_BUTTON_5_PINS,MASTER_BUTTON_5_PIN))
			(*target).buttons|=1<<5;
	}
  if(next_gamepad!=MASTER_GAMEPAD_ID)//the pins used to multiplex inputs must be updated
		SELECTOR_PORT= SELECTOR_PINS_MASK & ~(1<<(next_gamepad+1));
	usb_gamepad_send(GAMEPAD_INTERFACE(scaned_gamepad));
	scaned_gamepad=next_gamepad;
//update pseudo-random value
	random_value++;
	random_value&=3;
 }
