/* 4+1 Joystick plus Simon game
 * Copyright (C) 2015 David Guerrero
 * Based on Teensy Gamepad for Josh Kropf <josh@slashdev.ca>
 * Based on keyboard example for Teensy USB Development Board
 * http://www.pjrc.com/teensy/usb_keyboard.html
 * Copyright (c) 2008 PJRC.COM, LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef SIMON
#include "simon.h"
#endif
#ifndef SOUND
#include "sound.h"
#endif
#ifndef CENTRAL_BUTTONS
#include "central_buttons.h"
#endif
#ifndef TIMING
#include "timing.h"
#endif
#ifndef HD44780_H_INCLUDED
#include "HD44780.h"
#endif

uint16_t tones[6];
#define play_tone(x) beep(tones[x])
extern volatile uint8_t random_value;
uint8_t wrong_button(uint8_t button);

extern volatile uint8_t light_buttons_values;
extern volatile uint8_t simon_start_buttons_values;

//extern uint32_t credits;
////////////////////these were the original delays///////////////////
#define tone_duration1 420//miliseconds
#define tone_duration2 320//miliseconds
#define tone_duration3 220//miliseconds
#define pause_duration 50//miliseconds
#define pressing_tone_duration 400//miliseconds
#define time_out 3000//miliseconds
#define lossing_tone_duration 1500//miliseconds
#define intersequence_pause_duration 800//miliseconds
//////////////////////////////////////////////////////////////////

#define SERVICE_BUTTON_DDR DDRB
#define SERVICE_BUTTON_PORT PORTB
#define SERVICE_BUTTON_PINS PINB
#define SERVICE_BUTTON_PIN 6
#define SERVICE_MODE_DISABLED (SERVICE_BUTTON_PINS & (1<<SERVICE_BUTTON_PIN))

uint8_t sequence[31];

void configure_simon(void)
{
	configure_beeper();
	tones[0]=42;	//losing frequency in Hertzs
	tones[1]=252;		//yellow frequency in Hertzs
	tones[2]=209;	//blue frequency in Hertzs
	tones[3]=415;		//green frequency in Hertzs
	tones[4]=310;		//red frequency in Hertzs
	tones[5]=600;		//victory frequency in Hertzs
	SERVICE_BUTTON_PORT|=(1<<SERVICE_BUTTON_PIN);//activates pull-up
}

void play_button(uint8_t button_number)
{
	turn_off_color_button_lights;
	turn_on_color_button_light(button_number);
	play_tone(button_number+1);
}

void play_button_for(uint8_t button_number,uint16_t time)
{
	play_button(button_number);
	wait_for_miliseconds(time);
	turn_off_color_button_lights;
	nobeep;
}

void play_button_five_times(uint8_t button_number)
{
	 for(uint8_t i=0;i<5;i++)//repeat last tone five times
	 	{
	 		wait_for_miliseconds(20);
	 		play_button_for(button_number,100);
	 	}
}

void play_sequence(uint8_t sequence[],uint8_t length)
{
		uint16_t beep_duration= length<=5? tone_duration1 : length<=13? tone_duration2 : tone_duration1;
		for(uint8_t i=0;i<length;i++)
		{
			play_button_for(sequence[i],beep_duration);
			wait_for_miliseconds(pause_duration);
		}
}

uint8_t rigtht_sequence(uint8_t sequence[],uint8_t length)
{
		for(uint8_t i=0;i<length;i++)
			if(wrong_button(sequence[i]))
				return 0;
		return 1;
}

uint8_t wrong_button(uint8_t button)
{
	count_miliseconds(time_out);
  	while(pressed_any_light_button && !end_of_count);
	while(!end_of_count)
		if(pressed_any_light_button)
		{
			if(pressed_only_light_button(button))
			{
				count_miliseconds(pressing_tone_duration);
				play_button(button);
  				while(pressed_any_light_button && !end_of_count);
				turn_off_color_button_lights;
				nobeep;
				return 0;
			}
			else
				break;
		}
	LIGHTS_PORT|=(light_buttons_values<<1) & ~1;//right button not pressed
	play_tone(0);
	wait_for_miliseconds(lossing_tone_duration);
	play_button_five_times(button);
	return 1;
}

int32_t simon_game(uint8_t skill_level)
{
	//uint8_t previous_lights_value=LIGHTS_PORT;
	uint8_t current_length;
	uint8_t victory=1;
	//skill_level 0: sequence length=4
	//skill_level 1: sequence length=8
  //skill_level 2: sequence length=14
	//skill_level 3: sequence length=20
	//skill_level 4+: sequence length=31
	uint8_t sequence_length=4;
	if(skill_level)
		sequence_length= skill_level>=4? 31 : 2+6*skill_level;
	else//skill_level 0 (no credits)
		{
				for(uint8_t i=0;i<6;i++)
				{
					LCD_LED_TOGGLE;
					play_tone(0);
					wait_for_miliseconds(80);
					nobeep;
					wait_for_miliseconds(20);
				}
		}																														//skill_level 4 and above- sequence length=31
	LCD_Clear();
	LCD_ClearLine(0);
	LCD_ClearLine(1);
	LCD_GotoXY(0,0);
	LCD_PrintString("Simon level ");
	LCD_PrintInteger((int32_t)skill_level);
	turn_off_color_button_lights;
	if(SERVICE_MODE_DISABLED)
		for(current_length=0;victory && current_length<sequence_length;)
		{
			sequence[current_length++]=random_value;
			LCD_GotoXY(0,1);
			LCD_PrintString("round ");
			LCD_PrintInteger((int32_t)current_length);
			LCD_PrintString(" of ");
			LCD_PrintInteger((int32_t)sequence_length);
			play_sequence(sequence,current_length);
			if(!rigtht_sequence(sequence,current_length))
				victory=0;
			wait_for_miliseconds(intersequence_pause_duration);
		}
	if(victory)
	{
			for(uint8_t i=0;i<6;i++)
			{
				LCD_LED_TOGGLE;
				play_tone(5);
				wait_for_miliseconds(80);
				nobeep;
				wait_for_miliseconds(20);
			}
	}
	return (!victory)	? 0
										: skill_level>3? ((int32_t)(-1))-(((uint32_t)(-1))>>1)//the lowest negative number
										: skill_level>2? 500
										: skill_level>1? 50
										: skill_level>0? 5
										: 1;
}
